<?php
namespace Fiscal\Logic\ImportacaoAF;

use \Fiscal\Entities\Notafiscal;
use \Fiscal\Entities\Trabitproimpos;
use Fiscal\Logic\NotaFiscal\Entrada\NFEntrada;
use Fiscal\Logic\NotaFiscal\ProceduresNF;
use \Geral\Entities\Encargo;
use \Geral\Entities\Filial;
use Geral\Entities\Fornecedor;
use \Geral\Entities\Itemautf;
use \Geral\Entities\Itemproduto;
use \Geral\Entities\Moddocfisc;
use \Geral\Entities\Paramestoque;
use \Geral\Entities\Paramfilial;
use \Geral\Entities\Paramgeral;
use \Geral\Entities\Produto;
use Estoque\Util\Parametros;
use \Suprimentos\Entities\Encargoaf;
use \Suprimentos\Entities\Impitmaf;
use \Suprimentos\Entities\Nfrelacaf;
use \Setforms2\Runtime\Helper\Procedure\Param;
use \Setforms2\Runtime\Helper\Procedure;

use \Geral\Util\Ambiente;
use \Geral\Util\DateUtil;

/**
 * Created by JetBrains PhpStorm.
 * User: paulopereira
 * Date: 19/02/13
 * Time: 09:46
 * To change this template use File | Settings | File Templates.
 */
class ImportadorAF
{

    /**
     * Dados das AFs
     *
     * Variavel que armazenara todos os dados da AF, no formato de array multi dimensional, onde o primeiro indice
     * representa cada AF selecionada para a importação. No segundo nivel, os seguintes itens devem existir:
     * NRAUTOFORN, DTAUTOFORN, IDTPIJURFORN, DTPROGENTAF, NRINSJURFORN, CDFORNECED, FORNECEDOR, NMRAZSOCFORN,
     * CDOPERADOR, CDFILDEST, CDFILLANC, CDEMPRESA, NRNOTAFISC, NRLANCTONF, CDSERIEENTR.
     *
     * @var $dadosAFs array
     */
    private $dadosAFs;

    private $filialCorrente;

    /** @var $paramestoque Paramestoque*/
    private $paramestoque;
    /** @var $paramfilial Paramfilial */
    private $paramfilial;
    /** @var Notafiscal */
    private $notafiscal;

    /** Armazena a soma do valor de todos os produtos x a quantidade. */
    private $valorTotalProdutos;
    /*Armazena a soma do valor de todos os produtos x a quantidade, mais seus impostos.*/
    private $valorTotalNotafiscal;
    /**
     * Variavel auxiliar para os metodos de validacao dos dados da importacao.
     */
    private $idPilotoAnterior;

    /**
     * Construtor.
     *
     * @param array $dadosAF
     */
    public function __construct(array $dadosAF)
    {
        $this->dadosAFs         = $dadosAF;
        $this->idPilotoAnterior = "";
        $this->filialCorrente   = Ambiente::getInstancia()->getFilial();

        $this->inicializaParametros();
        $this->inicializaNotaFiscal();
    }


    // <editor-fold desc="Execucao da importacao de AF.">
    /**
     * Verifica se a AF e o fornecedor da AF estao validos para a importacao. Caso nao estejam, este retorna uma
     * excecao.
     *
     * @throws \Exception
     */
    public function validaImportacao()
    {
        try {
            $this->validaQuantidadeAf();
            foreach ($this->dadosAFs as $AF) {
                $this->validaAFsPiloto($AF["IDAFDESTALMOXPIL"]);
                $this->verificaFornecedorAtivo($AF["CDFORNECED"], Ambiente::getNrorg());
                $this->verificaQualificacaoFornecedor (
                    $AF["CDFORNECED"], Ambiente::getCdFilial(), $AF["DTAUTOFORN"], Ambiente::getNrorg()
                );
                $this->verificaEmpresaPertencenteAFilial (
                    $AF["CDEMPRESA"], Ambiente::getCdFilial(), Ambiente::getNrorg()
                );
                $this->verificaAlmoxarifado();
                $this->verificaLocalEstoque();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $cdserie
     * @param $nrnotafisc
     * @param $dtemissao
     * @param $dtentrada
     * @param $cdmoddocfisc
     * @param $cdtipooper
     * @param $nracessonfe
     */
    public function executaImportacao($cdserie, $nrnotafisc, $dtemissao, $dtentrada,
                                      $cdmoddocfisc, $cdtipooper, $nracessonfe)
    {
        try {
            \Setforms2\Shared\Registry::getEntityManager()->getConnection()->beginTransaction();
            $this->insereNotafiscal (
                $cdserie, $nrnotafisc, $dtemissao, $dtentrada, $cdmoddocfisc, $cdtipooper, $nracessonfe
            );
            $this->insereItemproduto();
            $this->atualizaValoresTotaisNota();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

            $this->geraNotafiscalEncargo();

            \Setforms2\Shared\Registry::getEntityManager()->getConnection()->commit();
        } catch (\Exception $e) {
            \Setforms2\Shared\Registry::getEntityManager()->getConnection()->rollBack();
            throw $e;
        }
    }

    public function executaImportacaoWiz()
    {
        try {
            \Setforms2\Shared\Registry::getEntityManager()->getConnection()->beginTransaction();

            $this->insereItemprodutoWiz();

            \Setforms2\Shared\Registry::getEntityManager()->flush();
            \Setforms2\Shared\Registry::getEntityManager()->getConnection()->commit();
        } catch (\Exception $e) {
            \Setforms2\Shared\Registry::getEntityManager()->getConnection()->rollBack();
            throw $e;
        }
    }
    //</editor-fold>


    //<editor-fold desc="Inicializacao de parametros da classe">
    /**
     * Inicializa objetos de parametros.
     */
    private function inicializaParametros()
    {
        $this->paramestoque = Paramestoque::findOneByNrorg(Ambiente::getNrorg());
        $this->paramfilial  = Paramfilial::findOneByCdfilialAndNrorg(Ambiente::getCdFilial(), Ambiente::getNrorg());
    }
    /**
     * Inicializa os itens utilizados para a nota fiscal.
     */
    private function inicializaNotaFiscal()
    {
        $this->notafiscal = new Notafiscal();
        $this->valorTotalProdutos   = 0;
        $this->valorTotalNotafiscal = 0;
    }
    //</editor-fold>


    //<editor-fold desc="Validacoes da importacao">
    /**
     * Verifica se ao menos uma AF foi selecionada para a importacao.
     *
     * @throws Exceptions\NenhumaAFSelecionada
     */
    private function validaQuantidadeAf()
    {
        if (count($this->dadosAFs) === 0) {
            throw new \Fiscal\Logic\ImportacaoAF\Exceptions\NenhumaAFSelecionada();
        }
    }

    /**
     * Executa a validacao de AFs piloto: O processo de importacao de AFs nao permite que ambas, AFs piloto e AFs nao
     * piloto, sejam incluidos em uma mesma nota fiscal. Este metodo verificara se, dentre as AFs informadas,
     * existe alguma.
     * IMPORTANTE: Esta é uma validação iterativa, ou seja, esta é executada dentro de um foreach. O resultado deste
     * metodo é influenciado pelas iterações anteriores.
     *
     * @param $idAFPiloto
     * @throws Exceptions\ImportacaoAFsPiloto
     */
    private function validaAFsPiloto($idAFPiloto)
    {
        // Verifica se o id de AF piloto e igual aos anteriores ou se e o primeiro verificado.
        if ($idAFPiloto !== $this->idPilotoAnterior && $this->idPilotoAnterior !== "") {
            throw new \Fiscal\Logic\ImportacaoAF\Exceptions\ImportacaoAFsPiloto();
        }

        $this->idPilotoAnterior = $idAFPiloto;
    }

    /**
     * Verifica se o fornecedor esta ativo.
     *
     * @param $cdforneced
     * @param $nrorg
     * @throws Exceptions\FornecedorInativo
     */
    private function verificaFornecedorAtivo($cdforneced, $nrorg)
    {
        if ($this->paramestoque->getIdbloqforirr() === Paramestoque::IDBLOQFORIRR_SIM) {
            $fornecedor = \Geral\Entities\Fornecedor::findOneByCdfornecedAndNrorg($cdforneced, $nrorg);
            if ($fornecedor->getIdsitucada() === \Geral\Entities\Fornecedor::ID_FORNECEDOR_INATIVO) {
                throw new \Fiscal\Logic\ImportacaoAF\Exceptions\FornecedorInativo();
            }
        }
    }

    /**
     * Verifica se o fornecedor esta qualificado para responder por AFs.
     *
     * @param $cdforneced
     * @param $cdfilial
     * @param $dtautoforn
     * @param $nrorg
     * @throws Exceptions\ForncedorDesqualificado
     */
    private function verificaQualificacaoFornecedor($cdforneced, $cdfilial, $dtautoforn, $nrorg)
    {
        /** @var $forncedorRepository \Geral\Repositories\Fornecedor */
        $forncedorRepository = \Geral\Entities\Fornecedor::getRepository();
        $status              = (bool)$forncedorRepository->getStatusDeQualificacaoFornecedorImportacaoAF(
            $cdforneced, $cdfilial, $dtautoforn, $nrorg
        );

        if ($status === false && $this->paramfilial->getIdperforneco() === Paramfilial::IDPERFORNECO_NAO) {
            throw new \Fiscal\Logic\ImportacaoAF\Exceptions\ForncedorDesqualificado();
        }
    }

    /**
     * Verifica se a empresa da AF pertence a filial a ser importada.
     *
     * @param $cdempresa
     * @param $cdfilial
     * @param $nrorg
     * @throws Exceptions\EmpresaNaoPertenceAFilial
     */
    private function verificaEmpresaPertencenteAFilial($cdempresa, $cdfilial, $nrorg)
    {
        /** @var $empresaRepositories \Geral\Repositories\Empresa */
        $empresaRepositories = \Geral\Entities\Empresa::getRepository();
        $empresaPertence     = (bool)$empresaRepositories->empresaPernteceAFilialDaImportacao (
            $cdempresa, $cdfilial, $nrorg
        );

        if ($empresaPertence === false) {
            throw new \Fiscal\Logic\ImportacaoAF\Exceptions\EmpresaNaoPertenceAFilial();
        }
    }

    /**
     * Verifica se a filial utiliza almoxarifado e, caso positivo, se há um almoxarifado cadastrado para esta.
     *
     * @throws \Fiscal\Logic\ImportacaoAF\Exceptions\AlmoxarifadoNaoCadastrado
     */
    private function verificaAlmoxarifado()
    {
        if ($this->paramfilial->getIdutilalmox() === Paramfilial::UTILIZA_ESTOQUE &&
            (bool)$this->paramfilial->getCdalmoxarife() === false) {
            throw new \Fiscal\Logic\ImportacaoAF\Exceptions\AlmoxarifadoNaoCadastrado();
        }
    }

    /**
     * Verifica se a filial utiliza localizacao de estoque e, caso positivo, se há um local estoque cadastrado para
     * esta.
     *
     * @throws Exceptions\LocalEstoqueNaoCadastrado
     */
    private function verificaLocalEstoque()
    {
        if ($this->paramfilial->getIdutillcestq() === Paramfilial::UTILIZA_LOCAL_ESTOQUE &&
            (bool)$this->paramfilial->getCdlocalestoq() === false) {
            throw new \Fiscal\Logic\ImportacaoAF\Exceptions\LocalEstoqueNaoCadastrado();
        }
    }
    //</editor-fold>


    //<editor-fold desc="insercao nas tabelas de nota fiscal.">
    /**
     * @param $cdserie
     * @param $nrnotafisc
     * @param $dtemissao
     * @param $dtentrada
     * @param $cdmoddocfisc
     * @param $cdtipooper
     * @param $nracessonfe
     */
    private function insereNotafiscal($cdserie, $nrnotafisc, $dtemissao, $dtentrada,
                                      $cdmoddocfisc, $cdtipooper, $nracessonfe)
    {
        /** @var $autoforne \Planejamento\Entities\Autoforne */
        $autoforne = \Planejamento\Entities\Autoforne::getObject(
            array(
                "cdfildest"  => $this->dadosAFs[0]["CDFILDEST"],
                "cdfillanc"  => $this->dadosAFs[0]["CDFILLANC"],
                "nrautoforn" => $this->dadosAFs[0]["NRAUTOFORN"],
                "nrorg"      => Ambiente::getNrorg(),
            )
        );
        if (($this->notafiscal instanceof Notafiscal) === false) {
            $this->inicializaNotaFiscal();
        }

        $this->notafiscal->setFilial(Filial::findOneByCdfilialAndNrorg(Ambiente::getCdFilial(), Ambiente::getNrorg()));
        $this->notafiscal->setNrlanctonf(Notafiscal::geraNovoNumeroLancamento());
        $this->notafiscal->setFilirece($autoforne->getFildest());
        $this->notafiscal->setCdempresa($this->filialCorrente->getCdempresa());
        $this->notafiscal->setInscesta($this->filialCorrente->getInscesta());
        $this->notafiscal->setIdtpijurnf($autoforne->getForneced()->getIdtpijurforn());
        $this->notafiscal->setNrinsjurnf($autoforne->getForneced()->getNrinsjurforn());
        $this->notafiscal->setForneced($autoforne->getForneced());
        $this->notafiscal->setModdocfisc(Moddocfisc::findOneByCdmoddocfiscAndNrorg($cdmoddocfisc, Ambiente::getNrorg()));
        $this->notafiscal->setIdbloqueado(Notafiscal::NOTA_FISCAL_NAO_BLOQUEADA);
        $this->notafiscal->setIddocfiscal(Notafiscal::DOCUMENTO_FISCAL);
        $this->notafiscal->setIddocfinanc(Notafiscal::DOCUMENTO_FINANCEIRO);
        $this->notafiscal->setNrnotafisc($nrnotafisc);
        $this->notafiscal->setDtemissao(DateUtil::getDataDeString($dtemissao));
        $this->notafiscal->setDtentrsaid(DateUtil::getDataDeString($dtentrada));
        $this->notafiscal->setVrprodutos(0);
        $this->notafiscal->setVrnotafisc(0);
        $this->notafiscal->setEntrsaidop(\Geral\Entities\Tipooperac::findOneByIdentrsaidopAndCdtipooperAndNrorg(
            \Geral\Entities\Tipooperac::OPERACAO_ENTRADA,
            $cdtipooper,
            Ambiente::getNrorg()
        ));
        $this->notafiscal->setCdserieentr($cdserie);
        $this->notafiscal->setIdstatus(Notafiscal::EM_DIGITACAO);
        $this->notafiscal->setIdsistema(Notafiscal::PREFIXO_SISTEMA_DFE);
        $this->notafiscal->setIdorigdestnf("F");
        $this->notafiscal->setDtlancamennf(DateUtil::getDataAtual());
        $this->notafiscal->setDtcompfatur(DateUtil::getDataAtual());
        $this->notafiscal->setNracessonfe($nracessonfe);
        // $this->notafiscal->setIdstatusnfe("???");
        $this->notafiscal->setIdimpressodanfe(Notafiscal::DOCUMENTO_NAO_IMPRESSO);
        $this->notafiscal->setIdtipoemisnfe(Notafiscal::IDTIPOEMISNFE_NORMAL); // ???
        $this->notafiscal->setIdexpnfep(Notafiscal::NFE_PAULISTANA_NAO);
        $this->notafiscal->setIdexpnfesanand(Notafiscal::IDEXPNFESANAND_NAO);
        $this->notafiscal->setIdexpnfemanaus(Notafiscal::NFE_MANAUS_NAO);
        $this->notafiscal->setIdemailnfe(Notafiscal::ENVIO_EMAIL_TRANSPORTADORA_PENDENTE);
        $this->notafiscal->setIdstatusctbnf(Notafiscal::STATUS_CONTABILIZACAO_PENDENTE);
        $this->notafiscal->setDtinclusao(DateUtil::getDataAtual());
        $this->notafiscal->setIdverifnfconf(Notafiscal::NOTA_NAO_CONFERIDA);
        $this->notafiscal->setIdnfdispfin(Notafiscal::DISPONIVEL_PARA_FINANCEIRO);
        $this->notafiscal->setIdtpambnfe(Notafiscal::IDTPAMBNFE_0);
        $this->notafiscal->setIdemailnfetransp(\Fiscal\Entities\Notafiscal::ENVIO_EMAIL_TRANSPORTADORA_PENDENTE);
        $this->notafiscal->setIdbloqnffin(Notafiscal::NF_LIBERADA_NO_FINANCEIRO);
        $this->notafiscal->setIdnfgeradaviaxml(Notafiscal::NF_NAO_GERADA_VIA_XML);
        $this->notafiscal->setIdnfcompraleg(Notafiscal::NF_COMPRA_LEGAL_NAO);
        $this->notafiscal->setIddebicrednf(\Fiscal\Entities\Notafiscal::ID_DEBITO_NENHUM_NOTA_FISCAL);
        $this->notafiscal->setIdstatusnfe(\Fiscal\Entities\Notafiscal::ID_STATUS_NFE_PENDENTE);

        $this->notafiscal->persist();
        /*@todo Espera-se que este flush possa ser removido, apos correcoes do setforms.*/
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    private function insereItemproduto()
    {
        // Variavel utilizada para gerar o codigo sequencial dos itens da nota.
        $nrseqitprod = 0;

        foreach ($this->dadosAFs as $arrayAF) {
            // encontrar todos os itens da AF.
            $itensAF = Itemautf::findByCdfildestAndCdfillancAndNrautofornAndNrorg (
                $arrayAF["CDFILDEST"],
                $arrayAF["CDFILLANC"],
                $arrayAF["NRAUTOFORN"],
                Ambiente::getNrorg()
            );

            /** @var $itemautf Itemautf */
            foreach ($itensAF as $itemautf) {
                $this->insereNfrelacaf($itemautf);

                $itemproduto = new Itemproduto();
                $itemproduto->setCdfilial($this->notafiscal->getCdfilial());
                $itemproduto->setNrlanctonf($this->notafiscal->getNrlanctonf());
                $itemproduto->setNrorg(Ambiente::getNrorg());
                $itemproduto->setNrseqitprod(str_pad(++$nrseqitprod, 4, "0", STR_PAD_LEFT));
                $itemproduto->setCdproduto($itemautf->getCdproduto());
                $itemproduto->setVrunititem($itemautf->getVrpreuniaf());
                $itemproduto->setIdconforitem(Itemproduto::ITEM_COM_CONFORMIDADE);
                $itemproduto->setDsprodemis($itemautf->getProduto()->getNmproduto());
                $itemproduto->setCdprodemis($itemautf->getProduto()->getCdproduto());
                $itemproduto->setQtitemprod (
                    floatval($itemautf->getQtprogentaf()) - floatval($itemautf->getQtefetentraf())
                );
                $itemproduto->setVrtotaitem($itemproduto->getVrunititem() * $itemproduto->getQtitemprod());
                $itemproduto->setIdatuvrestoq(Itemproduto::NAO_ATUALIZA_VALOR_ESTOQUE);
                $itemproduto->setVrdescitem($itemautf->getVrdescitmaf()?:0);
                $itemproduto->setVrbrutitem($itemautf->getVrpreuniaf());
                $itemproduto->setDtinclusao(DateUtil::getDataAtual());
                $itemproduto->setCdlocalestoq($this->filialCorrente->getEstoquePadrao());
                $itemproduto->setCdalmoxarife($this->filialCorrente->getAlmoxarifadoPadrao());
                $itemproduto->setCdoperador(Ambiente::getCdOperador());
                $itemproduto->setNrorgultatu(Ambiente::getNrorg());
                $itemproduto->setNrorginclusao(Ambiente::getNrorg());
                $itemproduto->setCdoperinclusao(Ambiente::getCdOperador());
                $itemproduto->setCdoperultatu(Ambiente::getCdOperador());
                $itemproduto->setDtultatu(DateUtil::getDataAtual());
                $itemproduto->setDtprogentaf($itemautf->getDtprogentaf());

                $itemproduto->setVrcontitem(0);
                $itemproduto->persist();

                /*@todo Espera-se que este flush possa ser removido, apos correcoes do setforms.*/
                \Setforms2\Shared\Registry::getEntityManager()->flush();

                $this->insereTrabitproimpos($itemproduto, $itemautf);
                $this->acumulaValorProdutos($itemproduto);
            }

            $this->acumulaValorNotaFiscal($arrayAF);
        }

        $this->valorTotalNotafiscal += $this->valorTotalProdutos;
    }

    private function insereItemprodutoWiz()
    {
        // Variavel utilizada para gerar o codigo sequencial dos itens da nota.
        $nrseqitprod = 0;

        foreach ($this->dadosAFs as $arrayAF) {
            // encontrar todos os itens da AF.
            $itensAF = Itemautf::findByCdfildestAndCdfillancAndNrautofornAndNrorg (
                $arrayAF["CDFILDEST"],
                $arrayAF["CDFILLANC"],
                $arrayAF["NRAUTOFORN"],
                Ambiente::getNrorg()
            );

            /** @var $itemautf Itemautf */
            foreach ($itensAF as $itemautf) {
                $this->insereNfrelacafWiz($itemautf, $arrayAF['NRLANCTONF']);

                $itemproduto = Itemproduto::findOneByCdfilialAndNrlanctonfAndNrorgAndCdproduto(Ambiente::getCdFilial(),
                    $arrayAF['NRLANCTONF'], Ambiente::getNrorg(), $itemautf->getCdproduto());

                if($itemproduto instanceof Itemproduto){
                    $oldQtitemprod = $itemproduto->getQtitemprod();
                    $itemproduto->setQtitemprod ($oldQtitemprod +
                            (floatval($itemautf->getQtprogentaf()) - floatval($itemautf->getQtefetentraf())));
                    $itemproduto->setVrtotaitem($itemproduto->getVrunititem() * $itemproduto->getQtitemprod());
                    $itemproduto->persist();

                    \Setforms2\Shared\Registry::getEntityManager()->flush();

                    $paramsBaixaAF = self::montaParamsBaixaAF($itemproduto, 'A', $oldQtitemprod);
                    ProceduresNF::callProcedureBaixaAF($paramsBaixaAF);

                    NFEntrada::setImpostosProduto($itemproduto);
                }

                $cdalmoxarife = $this->filialCorrente->getAlmoxarifadoPadrao();
                $cdlocalestoq = $this->filialCorrente->getEstoquePadrao();
                $nrloteestq = Parametros::getNumeroLote();
                $nrsublote = ' ';

                $cdalmoxarife = !empty($cdalmoxarife) ? $cdalmoxarife : ' ';
                $cdlocalestoq = !empty($cdlocalestoq) ? $cdlocalestoq : ' ';
                $nrloteestq = !empty($nrloteestq) ? $nrloteestq : ' ';

                $itemproduto = new Itemproduto();
                $itemproduto->setCdfilial(Ambiente::getCdFilial());
                $itemproduto->setCdfilimovi(Ambiente::getCdfilial());
                $itemproduto->setNrlanctonf($arrayAF['NRLANCTONF']);
                $itemproduto->setNrorg(Ambiente::getNrorg());
                $itemproduto->setNrseqitprod(str_pad(++$nrseqitprod, 4, "0", STR_PAD_LEFT));
                $itemproduto->setCdproduto($itemautf->getCdproduto());
                $itemproduto->setVrunititem($itemautf->getVrpreuniaf());
                $itemproduto->setIdconforitem(Itemproduto::ITEM_COM_CONFORMIDADE);
                $itemproduto->setDsprodemis($itemautf->getProduto()->getNmproduto());
                $itemproduto->setCdprodemis($itemautf->getProduto()->getCdproduto());
                $itemproduto->setQtitemprod (
                    floatval($itemautf->getQtprogentaf()) - floatval($itemautf->getQtefetentraf())
                );
                $itemproduto->setVrtotaitem($itemproduto->getVrunititem() * $itemproduto->getQtitemprod());
                $itemproduto->setIdatuvrestoq(Itemproduto::NAO_ATUALIZA_VALOR_ESTOQUE);
                $itemproduto->setVrdescitem($itemautf->getVrdescitmaf()?:0);
                $itemproduto->setVrbrutitem($itemautf->getVrpreuniaf());
                $itemproduto->setDtinclusao(DateUtil::getDataAtual());
                $itemproduto->setCdlocalestoq($cdlocalestoq);
                $itemproduto->setCdalmoxarife($cdalmoxarife);
                $itemproduto->setNrloteestq($nrloteestq);
                $itemproduto->setNrsublote($nrsublote);
                $itemproduto->setCdoperador(Ambiente::getCdOperador());
                $itemproduto->setNrorgultatu(Ambiente::getNrorg());
                $itemproduto->setNrorginclusao(Ambiente::getNrorg());
                $itemproduto->setCdoperinclusao(Ambiente::getCdOperador());
                $itemproduto->setCdoperultatu(Ambiente::getCdOperador());
                $itemproduto->setDtultatu(DateUtil::getDataAtual());
                $itemproduto->setDtprogentaf($itemautf->getDtprogentaf());

                $itemproduto->setVrcontitem(0);
                $itemproduto->persist();

                /*@todo Espera-se que este flush possa ser removido, apos correcoes do setforms.*/
                \Setforms2\Shared\Registry::getEntityManager()->flush();

                $paramsBaixaAF = self::montaParamsBaixaAF($itemproduto, 'I');
                ProceduresNF::callProcedureBaixaAF($paramsBaixaAF);

                NFEntrada::setImpostosProduto($itemproduto);

//                $this->insereTrabitproimpos($itemproduto, $itemautf);
//                $this->acumulaValorProdutos($itemproduto);
            }

//            $this->acumulaValorNotaFiscal($arrayAF);
        }

//        $this->valorTotalNotafiscal += $this->valorTotalProdutos;
    }

    public function insereItemprodutoAF(Itemproduto $itemproduto){
        foreach ($this->dadosAFs as $arrayAF) {
            $itemautf = Itemautf::findOneByCdfildestAndCdfillancAndNrautofornAndNrorgAndCdproduto(
                $arrayAF["CDFILDEST"],
                $arrayAF["CDFILLANC"],
                $arrayAF["NRAUTOFORN"],
                Ambiente::getNrorg(),
                $itemproduto->getCdproduto()
            );

            if($itemautf instanceof Itemautf){
                $this->insereNfrelacafWiz($itemautf, $arrayAF['NRLANCTONF']);

                $itemproduto->setCdmarca($itemautf->getCdmarca());
                $itemproduto->persist();
                \Setforms2\Shared\Registry::getEntityManager()->flush();

                $paramsBaixaAF = self::montaParamsBaixaAF($itemproduto, 'I');
                ProceduresNF::callProcedureBaixaAF($paramsBaixaAF);
                return;
            }
        }
    }

    public static function montaParamsBaixaAF(Itemproduto $itemproduto, $status, $oldQtitemprod = null, $oldCdprodemis = null, $oldCdproduto = null,
                                              $oldCdmarca = null){
        $paramsBaixaAF = array(
            'CDFILIAL' => $itemproduto->getCdfilial(),
            'NRLANCTONF' => $itemproduto->getNrlanctonf(),
            'NRSEQITPROD' => $itemproduto->getNrseqitprod(),
            'IDSTATUS' => $status,
            'NEWCDPRODEMIS' => $itemproduto->getCdprodemis(),
            'OLDCDPRODEMIS' => empty($oldCdprodemis) ? $itemproduto->getCdprodemis() : $oldCdprodemis,
            'NEWCDPRODUTO' => $itemproduto->getCdproduto(),
            'OLDCDPRODUTO' => empty($oldCdproduto) ? $itemproduto->getCdproduto() : $oldCdproduto,
            'NEWCDMARCA' => $itemproduto->getCdmarca(),
            'OLDCDMARCA' => empty($oldCdmarca) ? $itemproduto->getCdmarca() : $oldCdmarca,
            'NEWQTITEMPROD' => $itemproduto->getQtitemprod(),
            'OLDQTITEMPROD' => empty($oldQtitemprod) ? $itemproduto->getQtitemprod() : $oldQtitemprod
        );

        return $paramsBaixaAF;
    }

    /**
     * Acumula ao valor da nota fiscal os valores do encargo.
     *
     * @param array $arrayAF
     */
    private function acumulaValorNotaFiscal($arrayAF = array())
    {
        $encargosAF = Encargoaf::findByCdfildestAndCdfillancAndNrautofornAndNrorg(
            $arrayAF["CDFILDEST"],
            $arrayAF["CDFILLANC"],
            $arrayAF["NRAUTOFORN"],
            Ambiente::getNrorg()
        );

        /** @var $encargoaf Encargoaf*/
        foreach ($encargosAF as $encargoaf) {
            $cdencargo = $encargoaf->getCdencargo();
            $vrencargo = $encargoaf->getVrencargoaf();
            $encargo = Encargo::findOneByCdencargoAndNrorg($cdencargo, Ambiente::getNrorg());
            if ($encargo->getIdencargo() === Encargo::IDENCARGO_DESCONTO) {
                $vrencargo = -1 * $vrencargo;
            }

            $this->valorTotalNotafiscal += $vrencargo;
        }
    }

    /**
     * Insere dados na tabela Trabitproimpos.
     *
     * @param \Geral\Entities\Itemproduto $itemproduto
     * @param \Geral\Entities\Itemautf $itemautf
     */
    private function insereTrabitproimpos(Itemproduto $itemproduto, Itemautf $itemautf)
    {
        $where = " CDFILDEST        = ?
                    AND CDFILLANC   = ?
                    AND NRAUTOFORN  = ?
                    AND CDPRODUTO   = ?
                    AND CDMARCA     = ?
                    AND DTPROGENTAF = ?
                    AND NRORG       = ?";
        $params = array(
            $itemautf->getCdfildest(),
            $itemautf->getCdfillanc(),
            $itemautf->getNrautoforn(),
            $itemautf->getCdproduto(),
            $itemautf->getCdmarca(),
            $itemautf->getDtprogentaf()->format(DateUtil::FORMATO_BRASILEIRO),
            Ambiente::getNrorg(),
        );
        $arrayImpitmaf = Impitmaf::getObjectsByWhere($where, $params);

        /** @var $impitmaf Impitmaf */
        foreach ($arrayImpitmaf as $impitmaf) {
            $trabitproimpos = new Trabitproimpos();
            $trabitproimpos->setCdfilimpoit($itemproduto->getCdfilial());
            $trabitproimpos->setNrlancimpoit($itemproduto->getNrlanctonf());
            $trabitproimpos->setNrseqimpoit($itemproduto->getNrseqitprod());
            $trabitproimpos->setCdimpostoit($impitmaf->getCdimposto());
            $trabitproimpos->setVrbcalcimpoit($impitmaf->getVrbasecalcaf());
            $trabitproimpos->setVraliqimpoit($impitmaf->getVraliqitmaf());
            $trabitproimpos->setVrimposit($impitmaf->getVrimpitmaf());
            $trabitproimpos->setCdfiscimpoit($impitmaf->getCdfiscitmaf());
            $trabitproimpos->setNrseqitpro("1");
            $trabitproimpos->setIdvalorinfo("N");
            $trabitproimpos->setVrcontimpoit($itemproduto->getVrcontitem());
            $trabitproimpos->setNrorg(Ambiente::getNrorg());
            $trabitproimpos->setDtinclusao(DateUtil::getDataAtual());
            $trabitproimpos->setDtultatu(DateUtil::getDataAtual());
            $trabitproimpos->setNrorginclusao(Ambiente::getNrorg());
            $trabitproimpos->setCdoperinclusao(Ambiente::getCdOperador());
            $trabitproimpos->setNrorgultatu(Ambiente::getNrorg());
            $trabitproimpos->setCdoperultatu(Ambiente::getCdOperador());
            $trabitproimpos->setIddifaliqcuspti('N');
            $trabitproimpos->setIdimpdesotrabit('N');
            $trabitproimpos->setIdmotdesotrabit(9);
            $trabitproimpos->setVrredbcimpoit(0);
            $trabitproimpos->setVrpdedimpoit(0);

            $trabitproimpos->persist();
        }
        /*@todo Espera-se que este flush possa ser removido, apos correcoes do setforms.*/
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    /**
     * Insere um registro
     *
     * @param \Geral\Entities\Itemautf $itemautf
     */
    private function insereNfrelacaf(Itemautf $itemautf)
    {
        $nfrelacaf = new Nfrelacaf();

        $nfrelacaf->setCdfilial($this->notafiscal->getCdfilial());
        $nfrelacaf->setNrlanctonf($this->notafiscal->getNrlanctonf());
        $nfrelacaf->setCdfildest($itemautf->getCdfildest());
        $nfrelacaf->setCdfillanc($itemautf->getCdfillanc());
        $nfrelacaf->setNrautoforn($itemautf->getNrautoforn());
        $nfrelacaf->setCdmarca($itemautf->getCdmarca());
        $nfrelacaf->setCdproduto($itemautf->getCdproduto());
        $nfrelacaf->setDtprogentaf($itemautf->getDtprogentaf());
        $nfrelacaf->setNrorg(Ambiente::getNrorg());
        $nfrelacaf->setQtnfrelacaf(
            $itemautf->getVrpreuniaf() * floatval($itemautf->getQtprogentaf()) - floatval($itemautf->getQtsaidoplog())
        );
        $nfrelacaf->setVrnfrelacaf($itemautf->getVrpreuniaf());
        $nfrelacaf->setDtinclusao(DateUtil::getDataAtual());
        $nfrelacaf->setDtultatu(DateUtil::getDataAtual());
        $nfrelacaf->setNrorginclusao(Ambiente::getNrorg());
        $nfrelacaf->setCdoperinclusao(Ambiente::getCdOperador());
        $nfrelacaf->setNrorgultatu(Ambiente::getNrorg());
        $nfrelacaf->setCdoperultatu(Ambiente::getCdOperador());
        $nfrelacaf->persist();
        /*@todo Espera-se que este flush possa ser removido, apos correcoes do setforms.*/
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    private function insereNfrelacafWiz(Itemautf $itemautf, $nrlanctonf)
    {
        $nfrelacaf = new Nfrelacaf();

        $nfrelacaf->setCdfilial(Ambiente::getCdFilial());
        $nfrelacaf->setNrlanctonf($nrlanctonf);
        $nfrelacaf->setCdfildest($itemautf->getCdfildest());
        $nfrelacaf->setCdfillanc($itemautf->getCdfillanc());
        $nfrelacaf->setNrautoforn($itemautf->getNrautoforn());
        $nfrelacaf->setCdmarca($itemautf->getCdmarca());
        $nfrelacaf->setCdproduto($itemautf->getCdproduto());
        $nfrelacaf->setDtprogentaf($itemautf->getDtprogentaf());
        $nfrelacaf->setNrorg(Ambiente::getNrorg());
        $nfrelacaf->setQtnfrelacaf(
            $itemautf->getVrpreuniaf() * floatval($itemautf->getQtprogentaf()) - floatval($itemautf->getQtsaidoplog())
        );
        $nfrelacaf->setVrnfrelacaf($itemautf->getVrpreuniaf());
        $nfrelacaf->setDtinclusao(DateUtil::getDataAtual());
        $nfrelacaf->setDtultatu(DateUtil::getDataAtual());
        $nfrelacaf->setNrorginclusao(Ambiente::getNrorg());
        $nfrelacaf->setCdoperinclusao(Ambiente::getCdOperador());
        $nfrelacaf->setNrorgultatu(Ambiente::getNrorg());
        $nfrelacaf->setCdoperultatu(Ambiente::getCdOperador());
        $nfrelacaf->persist();
        /*@todo Espera-se que este flush possa ser removido, apos correcoes do setforms.*/
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }
    //</editor-fold>


    //<editor-fold desc="Demais Processamentos">

    /**
     * Atualiza os campos de VRPRODUTOS e VRNOTAFISC da nota fiscal.
     */
    private function atualizaValoresTotaisNota()
    {
        $this->notafiscal->setVrprodutos($this->valorTotalProdutos);
        $this->notafiscal->setVrnotafisc($this->valorTotalNotafiscal);
        $this->notafiscal->persist();
    }

    /**
     * Acumula o valor dos produtos x a quantidade dos produtos da nota.
     *
     * @param \Geral\Entities\Itemproduto $itemproduto
     */
    private function acumulaValorProdutos(Itemproduto $itemproduto)
    {
        $mensagem = $itemproduto->getCdproduto();
        $mensagem .= "\n".$itemproduto->getProduto()->getNmproduto();
        $mensagem .= "\n".floatval($itemproduto->getVrtotaitem());
        $mensagem .= "\n".floatval($itemproduto->getQtitemprod());
        $mensagem .= "\nGerado com sucesso!";
        $this->valorTotalProdutos += floatval($itemproduto->getVrtotaitem()) *
            floatval($itemproduto->getQtitemprod());
        tk_warning($mensagem);
    }

    /**
     * Executa a procedure GERA_NOTAFISCAL_ENCARGO para todos os encargos associados as AFs importadas.
     *
     * @throws \Exception
     */
    private function geraNotafiscalEncargo()
    {
        foreach ($this->dadosAFs as $AF) {
            /** @var $repositoryEncargoaf \Suprimentos\Repositories\Encargoaf */
            $repositoryEncargoaf = Encargoaf::getRepository();
            $encargosAF = $repositoryEncargoaf->getValorTotalEncargoAF (
                $AF["CDFILDEST"], $AF["CDFILLANC"], $AF["NRAUTOFORN"], Ambiente::getNrorg()
            );

            foreach ($encargosAF as $encargoAF) {
                try {
                    $this->executaProcedure($encargoAF);
                } catch (\Exception $e) {
                    throw $e;
                }
            }
        }
    }

    /**
     * Executa a procedure GERA_NOTAFISCAL_ENCARGO.
     *
     * @param $encargoAF
     * @throws \Exception
     */
    private function executaProcedure($encargoAF)
    {
        try {
            $procedure = new Procedure("GERA_NOTAFISCAL_ENCARGO");
            $procedure->addParam(new Param(
                "P_CDFILIAL",
                Param::PARAM_INPUT,
                $this->notafiscal->getCdfilial(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRLANCTONF",
                Param::PARAM_INPUT,
                $this->notafiscal->getNrlanctonf(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_CDENCARGO",
                Param::PARAM_INPUT,
                $encargoAF["CDENCARGO"],
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_VRITEMENCA",
                Param::PARAM_INPUT,
                $encargoAF["VALOR_ENCARGO"],
                Param::PARAM_TYPE_STR
            ));
            $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna os parametros para a query de AF
     *
     * @param Paramfilial $paramfilial
     */
    public static function montaParamAF(Paramfilial $paramfilial, $fornecedor){
        $nrmaxdiaantdtaf = $paramfilial->getNrmaxdiaantdtaf();
        $nrmaxdiaposdtaf = $paramfilial->getNrmaxdiaposdtaf();
        $dtprocessa = $paramfilial->getDtprocessa();

        $dtprocini = '';
        $dtprocfin = '';
        $dtmaxvazia = 'T';
        $dtminvazia = 'T';

        if ($nrmaxdiaantdtaf != null) {
            $dtprocini = DateUtil::subtraiIntervalo($dtprocessa, $nrmaxdiaantdtaf);
            $dtmaxvazia = '';
            $dtminvazia = '';
        }

        if ($nrmaxdiaposdtaf != null) {
            $dtprocfin = DateUtil::adicionaIntervalo($dtprocessa, $nrmaxdiaposdtaf);
            $dtmaxvazia = '';
            $dtminvazia = '';
        }

        $param = array(
            'CDFILIAL' => Ambiente::getCdFilial(),
            'CDFORNECED' => $fornecedor,
            'NRAUTOFORN' => '',
            'DTINICIAL' => '',
            'DTFINAL' => '',
            'DTTODAS' => 'T',
            'DTPROCINI' => $dtprocini,
            'DTPROCFIN' => $dtprocfin,
            'DTMAXVAZIA' => $dtmaxvazia,
            'IDBAIXAAF' => 'T',
            'LISTAAF' => '',
            'NRENTREGA' => '',
            'DTMINVAZIA' => $dtminvazia
        );
        return $param;
    }

    /**
     * Verifica se a AF é obrigatoria
     *
     * @param Paramgeral $paramgeral
     * @param Paramfilial $paramfilial
     * @param $nrorg
     * @param $af
     * @return bool
     */
    public static function verificaAFObrigatoria(Paramgeral $paramgeral, Paramfilial $paramfilial, $nrorg){
        $afObrigatoriaGeral = $paramgeral->getIdbloqcmpsaf();

        if($afObrigatoriaGeral == 'S')
            return true;

        $afObrigatoriaFilial = substr($paramfilial->getDsparfilfis(), 1, 1);

        if ($afObrigatoriaFilial == 'S')
            return true;

        $afObrigatoriaFornecedor = $paramfilial->getIdbloqcmpsaffil();

        if ($afObrigatoriaFornecedor == 'S') {
            /** @var $fornecedor Fornecedor */
            $fornecedor = Fornecedor::findOneByCdfornecedAndNrorg($fornecedor, $nrorg);

            if ($fornecedor->getIdbloqcmpsaffor() == 'S') {
                return true;
            }
        }

        return false;
    }

    //</editor-fold>

}
